@@ -27,6 +27,7 @@
 #define _GNU_SOURCE
 #include <string.h>
 #include <stdarg.h>
+#include <stdio.h>
 #include <fcntl.h>
 #include <sys/types.h>
 #include <sys/stat.h>
@@ -141,7 +142,6 @@ ssize_t write_file(const void *buffer, size_t len,
 		goto error_mkostemps;
 
 	r = TFR(write(fd, buffer, len));
-
 	TFR(close(fd));
 
 	if (r != (ssize_t) len) {
@@ -153,15 +153,15 @@ ssize_t write_file(const void *buffer, size_t len,
 	 * Now that the file contents are written, rename to the real
 	 * file name; this way we are uniquely sure that the whole
 	 * thing is there.
+	 * conserve @r's value from 'write'
 	 */
-	unlink(path);
 
-	/* conserve @r's value from 'write' */
-	if (link(tmp_path, path) == -1)
+	if (rename(tmp_path, path) == -1)
 		r = -1;
 
 error_write:
-	unlink(tmp_path);
+	if (r < 0)
+		unlink(tmp_path);
 error_mkostemps:
 error_create_dirs:
 	l_free(tmp_path);
@@ -169,7 +169,7 @@ error_create_dirs:
 	return r;
 }
 
-static char *get_network_file_path(const char *type, const char *ssid)
+char *storage_get_network_file_path(enum security type, const char *ssid)
 {
 	char *path;
 	const char *c;
@@ -183,11 +183,13 @@ static char *get_network_file_path(const char *type, const char *ssid)
 		hex = l_util_hexstring((const unsigned char *) ssid,
 					strlen(ssid));
 
-		path = l_strdup_printf(STORAGEDIR "/=%s.%s", hex, type);
+		path = l_strdup_printf(STORAGEDIR "/=%s.%s", hex,
+					security_to_str(type));
 
 		l_free(hex);
 	} else
-		path = l_strdup_printf(STORAGEDIR "/%s.%s", ssid, type);
+		path = l_strdup_printf(STORAGEDIR "/%s.%s", ssid,
+					security_to_str(type));
 
 	return path;
 }
@@ -250,46 +252,35 @@ const char *storage_network_ssid_from_path(const char *path,
 	return buf;
 }
 
-struct l_settings *storage_network_open(const char *type, const char *ssid)
+struct l_settings *storage_network_open(enum security type, const char *ssid)
 {
 	struct l_settings *settings;
-	struct stat st;
 	char *path;
 
-	if (ssid == NULL || type == NULL)
+	if (ssid == NULL)
 		return NULL;
 
-	path = get_network_file_path(type, ssid);
-
-	if (lstat(path, &st) < 0) {
-		l_free(path);
-
-		return NULL;
-	}
-
+	path = storage_get_network_file_path(type, ssid);
 	settings = l_settings_new();
 
 	if (!l_settings_load_from_file(settings, path)) {
-		l_free(path);
 		l_settings_free(settings);
-
-		return NULL;
+		settings = NULL;
 	}
 
 	l_free(path);
-
 	return settings;
 }
 
-int storage_network_touch(const char *type, const char *ssid)
+int storage_network_touch(enum security type, const char *ssid)
 {
 	char *path;
 	int ret;
 
-	if (ssid == NULL || type == NULL)
+	if (ssid == NULL)
 		return -EINVAL;
 
-	path = get_network_file_path(type, ssid);
+	path = storage_get_network_file_path(type, ssid);
 	ret = utimensat(0, path, NULL, 0);
 	l_free(path);
 
@@ -299,17 +290,17 @@ int storage_network_touch(const char *type, const char *ssid)
 	return -errno;
 }
 
-int storage_network_get_mtime(const char *type, const char *ssid,
+int storage_network_get_mtime(enum security type, const char *ssid,
 				struct timespec *mtim)
 {
 	char *path;
 	int ret;
 	struct stat sb;
 
-	if (ssid == NULL || type == NULL)
+	if (ssid == NULL)
 		return -EINVAL;
 
-	path = get_network_file_path(type, ssid);
+	path = storage_get_network_file_path(type, ssid);
 	ret = stat(path, &sb);
 	l_free(path);
 
@@ -325,26 +316,26 @@ int storage_network_get_mtime(const char *type, const char *ssid,
 	return 0;
 }
 
-void storage_network_sync(const char *type, const char *ssid,
+void storage_network_sync(enum security type, const char *ssid,
 				struct l_settings *settings)
 {
 	char *data;
 	size_t length = 0;
 	char *path;
 
-	path = get_network_file_path(type, ssid);
+	path = storage_get_network_file_path(type, ssid);
 	data = l_settings_to_data(settings, &length);
 	write_file(data, length, "%s", path);
 	l_free(data);
 	l_free(path);
 }
 
-int storage_network_remove(const char *type, const char *ssid)
+int storage_network_remove(enum security type, const char *ssid)
 {
 	char *path;
 	int ret;
 
-	path = get_network_file_path(type, ssid);
+	path = storage_get_network_file_path(type, ssid);
 	ret = unlink(path);
 	l_free(path);
 
